/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.core;

import javax.annotation.Nonnull;

import org.apache.jackrabbit.oak.plugins.version.VersionConstants;
import org.apache.jackrabbit.oak.spi.security.Context;
import org.apache.jackrabbit.oak.spi.security.authorization.permission.PermissionConstants;
import org.apache.jackrabbit.oak.spi.state.NodeStateUtils;

/**
 * Default implementation of the {@link TreeTypeProvider} interface.
 */
public final class TreeTypeProviderImpl implements TreeTypeProvider {

    private final Context contextInfo;

    public TreeTypeProviderImpl(@Nonnull Context contextInfo) {
        this.contextInfo = contextInfo;
    }

    @Override
    public int getType(ImmutableTree tree) {
        if (tree.isRoot()) {
            return TYPE_DEFAULT;
        }

        ImmutableTree parent = tree.getParent();
        int type;
        switch (parent.getType()) {
            case TYPE_HIDDEN:
                type = TYPE_HIDDEN;
                break;
            case TYPE_VERSION:
                type = TYPE_VERSION;
                break;
            case TYPE_INTERNAL:
                type = TYPE_INTERNAL;
                break;
            case TYPE_AC:
                type = TYPE_AC;
                break;
            default:
                String name = tree.getName();
                if (NodeStateUtils.isHidden(name)) {
                    type = TYPE_HIDDEN;
                } else if (VersionConstants.VERSION_STORE_ROOT_NAMES.contains(name)) {
                    type = TYPE_VERSION;
                } else if (PermissionConstants.REP_PERMISSION_STORE.equals(name)) {
                    type = TYPE_INTERNAL;
                } else if (contextInfo.definesContextRoot(tree)) {
                    type = TYPE_AC;
                } else {
                    type = TYPE_DEFAULT;
                }
        }
        return type;
    }
}
